@@ -1,7 +1,7 @@
 """Manifest init."""
 from manifest.manifest import Manifest
-from manifest.prompt import Prompt
+from manifest.request import Request
 from manifest.response import Response
 from manifest.session import Session
 
-__all__ = ["Manifest", "Prompt", "Response", "Session"]
+__all__ = ["Manifest", "Response", "Session"]
